/* GtkDatabox - An extension to the gtk+ library
 * Copyright (C) 1998-2003 Dr. Roland Bock
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <stdio.h>
#include <gtk/gtk.h>
#include <gtk/gtkcolorsel.h>
#include <gtkdatabox.h>
#include <math.h>
/* Some versions of math.h have a PI problem... */
#ifndef PI
#define PI 3.14159265358979323846
#endif

#define POINTS 2000

static void menu_color_change_cb (gpointer callback_data,
				  guint callback_action, GtkWidget * widget);

static GtkItemFactoryEntry menu_main_entries[] = {
   {"/_Color", NULL, NULL, 0, "<Branch>"},
   {"/Color/Change Color #0", NULL, menu_color_change_cb, 0, NULL},
   {"/Color/Change Color #1", NULL, menu_color_change_cb, 1, NULL},
   {"/Color/Change Color #2", NULL, menu_color_change_cb, 2, NULL},
   {"/Color/Change Color #3", NULL, menu_color_change_cb, 3, NULL},
   {"/Color/Change Color #4", NULL, menu_color_change_cb, 4, NULL}
};

static gint
   menu_main_num = sizeof (menu_main_entries) / sizeof (menu_main_entries[0]);

typedef struct
{
   GtkWidget *selector;
   GtkWidget *box;
   gint index;
}
col_sel;

static void
get_color_cb (GtkWidget * widget, col_sel * sel)
{
   GdkColor color;

   g_return_if_fail (GTK_IS_COLOR_SELECTION_DIALOG (sel->selector));
   g_return_if_fail (GTK_IS_DATABOX (sel->box));

   gtk_color_selection_get_current_color (GTK_COLOR_SELECTION
					  (GTK_COLOR_SELECTION_DIALOG
					   (sel->selector)->colorsel),
					  &color);

   gtk_databox_data_set_color (GTK_DATABOX (sel->box), sel->index, color);
   gtk_databox_redraw (GTK_DATABOX (sel->box));
}

static void
menu_color_change_cb (gpointer callback_data, guint callback_action,
		      GtkWidget * widget)
{
   GtkWidget *selector;
   gchar title[20];
   col_sel *sel = g_new0 (col_sel, 1);
   GdkColor color;

   sprintf (title, "Choose color #%d", callback_action);
   selector = gtk_color_selection_dialog_new (title);
   gtk_widget_destroy (GTK_COLOR_SELECTION_DIALOG (selector)->help_button);
   sel->selector = selector;
   sel->box = GTK_WIDGET (callback_data);
   sel->index = callback_action;

   gtk_databox_data_get_color (GTK_DATABOX (sel->box), sel->index, &color);
   gtk_color_selection_set_current_color (GTK_COLOR_SELECTION
					  (GTK_COLOR_SELECTION_DIALOG
					   (sel->selector)->colorsel),
					  &color);

   g_signal_connect_object (GTK_OBJECT
			    (GTK_COLOR_SELECTION_DIALOG (selector)->
			     cancel_button), "clicked",
			    G_CALLBACK (gtk_widget_destroy),
			    G_OBJECT (selector), G_CONNECT_SWAPPED);
   g_signal_connect (GTK_OBJECT
		     (GTK_COLOR_SELECTION_DIALOG (selector)->ok_button),
		     "clicked", G_CALLBACK (get_color_cb), (gpointer) sel);
   gtk_widget_show (selector);

   return;
}

static void
create_colors (void)
{
   static GtkWidget *window = NULL;
   GtkWidget *box1 = NULL;
   GtkWidget *box2 = NULL;
   GtkWidget *close_button = NULL;
   GtkWidget *box = NULL;
   GtkWidget *separator;
   GtkWidget *label;
   GtkWidget *menu_bar;
   GtkItemFactory *item_factory;
   gfloat *X = NULL;
   gfloat *Y = NULL;
   gint i, j;
   GdkColor color;
   gint index = 0;
   gchar title[] = "<Main>";

   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_widget_set_size_request (window, 500, 300);

   g_signal_connect (GTK_OBJECT (window), "destroy",
		     G_CALLBACK (gtk_main_quit), NULL);

   gtk_window_set_title (GTK_WINDOW (window), "GtkDatabox: Colors");
   gtk_container_set_border_width (GTK_CONTAINER (window), 0);

   box1 = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER (window), box1);

   box = gtk_databox_new ();
   color.red = 0;
   color.green = 0;
   color.blue = 0;
   gtk_databox_set_background_color (GTK_DATABOX (box), color);

   item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, title, NULL);
   gtk_item_factory_create_items (item_factory, menu_main_num,
				  menu_main_entries, box);
   menu_bar = gtk_item_factory_get_widget (item_factory, title);
   gtk_box_pack_start (GTK_BOX (box1), menu_bar, FALSE, TRUE, 0);

   label =
      gtk_label_new
      ("You can change the colors of the shown datasets via the menu.\n\n");
   gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 0);
   separator = gtk_hseparator_new ();
   gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, FALSE, 0);

   g_signal_connect (GTK_OBJECT (box), "destroy",
		     G_CALLBACK (gtk_main_quit), NULL);


   for (i = 0; i < 5; i++)
   {
      if (!i)
	 X = g_new0 (gfloat, POINTS);
      Y = g_new0 (gfloat, POINTS);
      for (j = 0; j < POINTS; j++)
      {
	 X[j] = j;
	 Y[j] = 100. * sin ((i + 1) * 2 * j * PI / POINTS);
      }
      color.red = 65535 * ((i + 1) % 2);
      color.green = (65535 / 2) * ((i + 1) % 3);
      color.blue = (65535 / 3) * ((i + 1) % 4);
      if (!i)
	 index = gtk_databox_data_add_x_y (GTK_DATABOX (box), POINTS,
					   X, Y, color,
					   GTK_DATABOX_POINTS, 0);
      else
	 gtk_databox_data_add_y (GTK_DATABOX (box), POINTS,
				 Y, index, color, GTK_DATABOX_POINTS, 0);
   }

   gtk_databox_rescale (GTK_DATABOX (box));

   gtk_box_pack_start (GTK_BOX (box1), box, TRUE, TRUE, 0);

   separator = gtk_hseparator_new ();
   gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);

   box2 = gtk_vbox_new (FALSE, 10);
   gtk_container_set_border_width (GTK_CONTAINER (box2), 10);
   gtk_box_pack_end (GTK_BOX (box1), box2, FALSE, TRUE, 0);
   close_button = gtk_button_new_with_label ("close");
   g_signal_connect (GTK_OBJECT (close_button), "clicked",
		     G_CALLBACK (gtk_main_quit), (gpointer) NULL);
   gtk_box_pack_start (GTK_BOX (box2), close_button, TRUE, TRUE, 0);
   GTK_WIDGET_SET_FLAGS (close_button, GTK_CAN_DEFAULT);
   gtk_widget_grab_default (close_button);

   gtk_widget_show_all (window);

}

gint
main (gint argc, char *argv[])
{
   gtk_init (&argc, &argv);

   create_colors ();
   gtk_main ();

   return 0;
}
